@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: sep  1 2020 (14:58) 
 ## Version: 
-## Last-Updated: sep 21 2020 (11:57) 
+## Last-Updated: Dec 21 2021 (12:13) 
 ##           By: Brice Ozenne
-##     Update #: 273
+##     Update #: 370
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -16,9 +16,9 @@
 ### Code:
 
 ## * wglm
-#' @title Logistic Regression Using IPCW 
+#' @title Logistic Regression Using IPCW
 #' @description Logistic regression over multiple timepoints
-#' where right-censoring is handle using inverse probability of censoring weighting.
+#' where right-censoring is handle using inverse probability of censoring weighting. 
 #'
 #' @param regressor.event [formula] a formula with empty left hand side and the covariates for the logistic regression on the right hand side.
 #' @param formula.censor [formula] a formula used to fit the censoring model.
@@ -42,25 +42,44 @@
 #' n <- 250
 #' tau <- 1:5
 #' d <- sampleData(n, outcome = "competing.risks")
-#' d$Y <- (d$event == 1)*(d$time <= tau[3])
-#' d0 <- d[event!=0] ## remove censoring
+#' dFull <- d[event!=0] ## remove censoring
+#' dSurv <- d[event!=2] ## remove competing risk
 #'
-#' ## no censoring
-#' e0.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ X1,
-#'                times = tau, data = d0)
-#' e0.glm <- glm(Y ~ X1, family = binomial, data = d0)
+#' #### no censoring ####
+#' e.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ 1,
+#'                times = tau, data = dFull, product.limit = TRUE)
+#' e.wglm ## same as a logistic regression
 #'
-#' ## censoring
-#' e.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ X1,
-#'                times = tau, data = d)
+#' summary(ate(e.wglm, data = dFull, times = tau, treatment = "X1", verbose = FALSE))
+#'
+#' #### right-censoring ####
+#' ## no covariante in the censoring model (independent censoring)
+#' eC.wglm <- wglm(regressor.event = ~ X1, formula.censor = Surv(time,event==0) ~ 1,
+#'                times = tau, data = dSurv, product.limit = TRUE)
+#' eC.wglm
+#'
+#' ## with covariates in the censoring model
+#' eC2.wglm <- wglm(regressor.event = ~ X1 + X8,
+#'                 formula.censor = Surv(time,event==0) ~ X1*X8,
+#'                  times = tau, data = dSurv)
+#' eC2.wglm
+#' 
+#' #### Competing risks ####
+#' ## here Kaplan-Meier as censoring model
+#' eCR.wglm <- wglm(regressor.event = ~ X1,
+#'                  formula.censor = Surv(time,event==0) ~ strata(X1),
+#'                  times = tau, data = d, cause = 1, product.limit = TRUE)
+#' eCR.wglm
+#' 
 #' @export
 wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
                  fitter = "coxph", product.limit = FALSE){
     
     tol <- 1e-12
     varSurv <- SurvResponseVar(formula.censor)
-    newname <- paste0(varSurv$status,".",times)
-    IPCW.newname <- paste0("IPCW.",times)
+    newname <- paste0("XX_",varSurv$status,".",times,"_XX")
+    obs.newname <- paste0("XX_observed.",times,"_XX")
+    IPCW.newname <- paste0("XX_IPCW.",times,"_XX")
     n.times <- length(times)
     
     ## ** check arguments
@@ -69,6 +88,10 @@ wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
         stop("Argument \'data\' should not have a column named \"",paste0(newname[newname %in% names(data)],collapse="\" \""),"\"\n",
              "This name is used internally \n.")
     }
+    if(any(obs.newname %in% names(data))){
+        stop("Argument \'data\' should not have a column named \"",paste0(obs.newname[obs.newname %in% names(data)],collapse="\" \""),"\"\n",
+             "This name is used internally \n.")
+    }
     if(any(IPCW.newname %in% names(data))){
         stop("Argument \'data\' should not have a column named \"",paste0(IPCW.newname[IPCW.newname %in% names(data)],collapse="\" \""),"\"\n",
              "This name is used internally \n.")
@@ -78,7 +101,7 @@ wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
     }
 
     ## ** fit censoring model
-    object.censor <- do.call(fitter, list(formula = formula.censor,data = data, x=TRUE,y=TRUE))
+    object.censor <- do.call(fitter, list(formula = formula.censor, data = data, x=TRUE,y=TRUE))
 
     ## ** extract information
     recoverCensor <- object.censor$y[,"status"]
@@ -95,17 +118,19 @@ wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
     for(iTime in 1:n.times){
         iFormula.glm <- update(regressor.event, paste0(newname[iTime],"~."))
         data[[newname[iTime]]] <- (data[[varSurv$status]]==cause)*(recoverTime<=times[iTime])
+        data[[obs.newname[iTime]]] <- (1-(recoverCensor==1)*(recoverTime<=times[iTime])) ## equivalent to status>0 or eventtime>tau
 
         if(n.censor[iTime]>0){
             iPred <- predictRisk(object.censor, diag = TRUE, newdata = data, times = pmin(recoverTime, times[iTime]) - tol,
                                  type = "survival", product.limit = product.limit)
-            data[[IPCW.newname[[iTime]]]] <- (1-(recoverCensor==1)*(recoverTime<times[iTime]))/iPred[,1]
-        
-            suppressWarnings(out$fit[[iTime]] <- do.call(stats::glm, list(formula = iFormula.glm, family = stats::binomial(link = "logit"),
+            data[[IPCW.newname[[iTime]]]] <- ifelse(data[[obs.newname[iTime]]],1/iPred[,1],0)
+            out$fit[[iTime]] <- suppressWarnings(do.call(stats::glm, list(formula = iFormula.glm, family = stats::binomial(link = "logit"),
                                                                           data = data, weights = data[[IPCW.newname[[iTime]]]])))
             ## Warning message:
             ##             In eval(family$initialize) : non-integer #successes in a binomial glm!
+
             out$fit[[iTime]]$time.prior.weights <- pmin(recoverTime, times[iTime]) - tol
+            out$fit[[iTime]]$prior.weights2 <- ifelse(data[[obs.newname[iTime]]],1/iPred[,1]^2,0)
         }else{
             IPCW.newname[[iTime]] <- NA
             out$fit[[iTime]] <- do.call(stats::glm, list(formula = iFormula.glm,
@@ -133,15 +158,18 @@ wglm <- function(regressor.event, formula.censor, times, data, cause = NA,
 }
 
 ## * nobs.wglm
-nobs.wglm <- function(object){
+#' @export
+nobs.wglm <- function(object, ...){
     return(object$n)
 }
 ## * formula.wglm
-formula.wglm <- function(object){
-    return(object$formula)
+#' @export
+formula.wglm <- function(x, ...){
+    return(x$formula)
 }
 
 ## * coef.wglm
+#' @export
 coef.wglm <- function(object, times = NULL, simplifies = TRUE, ...){
     if(is.null(times)){
         times <- object$times
@@ -159,6 +187,7 @@ coef.wglm <- function(object, times = NULL, simplifies = TRUE, ...){
 }
 
 ## * summary.wglm
+#' @export
 summary.wglm <- function(object, print = TRUE, se = "robust", times = NULL, ...){
 
     se <- match.arg(se, c("robust","model-wknown","robust-wknown"))
@@ -197,12 +226,12 @@ summary.wglm <- function(object, print = TRUE, se = "robust", times = NULL, ...)
         suppressWarnings(out[[iTime]] <- summary(object$fit[[iTime2]])$coef)
         if(se %in% c("robust","robust-wknown")){
             out[[iTime]][,"Std. Error"] <- sqrt(diag(crossprod(object.iid[[iTime]])))
-            out[[iTime]][,"z value"] <- out[[iTime]][,"Estimate"]/out[[iTime]][,"Std. Error"]
-            out[[iTime]][,"Pr(>|z|)"] <- 2*(1-pnorm(abs(out[[iTime]][,"z value"])))
+            out[[iTime]][,3] <- out[[iTime]][,"Estimate"]/out[[iTime]][,"Std. Error"] ## name change from z to t stat when using quasibinomial instead binomial
+            out[[iTime]][,4] <- 2*(1-pnorm(abs(out[[iTime]][,3])))
         }
         if(print){
             cat("----------------------------------------------------------------------------------\n")
-            cat("  > time: ",times[iTime],"\n",sep="")
+            cat("  - time: ",times[iTime],"\n",sep="")
             if(!is.na(eval(object$name.IPCW[[iTime2]]))){                
                 print(as.call(list(quote(glm),eval(object$fit[[iTime2]]$call$formula),family = quote(binomial(link="logit")), weights = eval(object$name.IPCW[[iTime2]]))))
             }else{
@@ -218,11 +247,12 @@ summary.wglm <- function(object, print = TRUE, se = "robust", times = NULL, ...)
 }
 
 ## * print.wglm
-print.wglm <- function(object, times = NULL, ...){
+#' @export
+print.wglm <- function(x, times = NULL, ...){
     if(is.null(times)){
-        times <- object$times
+        times <- x$times
     }else{
-        if(any(times %in% object$times == FALSE)){
+        if(any(times %in% x$times == FALSE)){
             stop("Incorrect specification of argument \'times\' \n",
                  "Should be one of \"",paste0(times,collapse="\" \""),"\" \n")
             
@@ -230,27 +260,27 @@ print.wglm <- function(object, times = NULL, ...){
     }
     n.times <- length(times)
     
-    if(length(object$causes)>1){
-        text.CR <- paste0("for cause ",object$theCause)
+    if(length(x$causes)>1){
+        text.CR <- paste0("for cause ",x$theCause)
     }else{
         text.CR <- ""
     }
-    if(any(object$n.censor>0)){
+    if(any(x$n.censor>0)){
         text.IPCW <- "IPCW "
     }else{
         text.IPCW <- ""
     }
     cat("     ",text.IPCW,"logistic regression ",text.CR,": \n",sep="")
-    for(iTime in which(times %in% object$times)){
-        iTime2 <- which(object$times == times[iTime])
-        if(!is.na(eval(object$name.IPCW[[iTime2]]))){                
-            object$fit[[iTime2]]$call <- as.call(list(quote(glm),eval(object$fit[[iTime2]]$call$formula),family = quote(binomial(link="logit")), weights = eval(object$name.IPCW[[iTime2]])))
+    for(iTime in which(times %in% x$times)){
+        iTime2 <- which(x$times == times[iTime])
+        if(!is.na(eval(x$name.IPCW[[iTime2]]))){                
+            x$fit[[iTime2]]$call <- as.call(list(quote(glm),eval(x$fit[[iTime2]]$call$formula),family = quote(binomial(link="logit")), weights = eval(x$name.IPCW[[iTime2]])))
         }else{
-            object$fit[[iTime2]]$call <- as.call(list(quote(glm),eval(object$fit[[iTime2]]$call$formula),family = quote(binomial(link="logit"))))
+            x$fit[[iTime2]]$call <- as.call(list(quote(glm),eval(x$fit[[iTime2]]$call$formula),family = quote(binomial(link="logit"))))
         }
         cat("----------------------------------------------------------------------------------\n")
-        cat("  > time: ",times[iTime],"\n",sep="")
-        print(object$fit[[iTime2]])
+        cat("  - time: ",times[iTime],"\n",sep="")
+        print(x$fit[[iTime2]])
         cat("----------------------------------------------------------------------------------\n")
     }
 }
@@ -264,10 +294,13 @@ print.wglm <- function(object, times = NULL, ...){
 #' @param times [numeric vector] time points at which the score should be output. 
 #' @param simplifies [logical] should the ouput be converted to a matrix when only one timepoint is requested. Otherwise will always return a list.
 #' @param ... Not used.
-#'
 #' @export
 score.wglm <- function(x, indiv = FALSE, times = NULL, simplifies = TRUE, ...){
-    if(is.null(times)){
+    if(inherits(x,"glm") && !inherits(x,"wglm")){
+        x <- list(fit = list("1" = x),
+                  times = "1")
+        times <- "1"
+    }else if(is.null(times)){
         times <- x$times
     }else{
         if(any(times %in% x$times == FALSE)){
@@ -307,10 +340,13 @@ score.wglm <- function(x, indiv = FALSE, times = NULL, simplifies = TRUE, ...){
 #' @param times [numeric vector] time points at which the score should be output. 
 #' @param simplifies [logical] should the ouput be converted to a matrix when only one timepoint is requested. Otherwise will always return a list.
 #' @param ... Not used.
-#' 
 #' @export
 information.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
-    if(is.null(times)){
+    if(inherits(x,"glm") && !inherits(x,"wglm")){
+        x <- list(fit = list("1" = x),
+                  times = "1")
+        times <- "1"
+    }else if(is.null(times)){
         times <- x$times
     }else{
         if(any(times %in% x$times == FALSE)){
@@ -349,10 +385,15 @@ information.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
 #' @param times [numeric vector] time points at which the iid should be output. 
 #' @param simplifies [logical] should the ouput be converted to a matrix when only one timepoint is requested. Otherwise will always return a list.
 #' @param ... Not used.
-#' 
 #' @export
 iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
-    if(is.null(times)){
+    if(inherits(x,"glm") && !inherits(x,"wglm")){
+        x <- list(fit = list("1" = x),
+                  times = "1",
+                  n.censor = 0)
+        times <- "1"
+        class(x) <- append("wglm",class(x))
+    }else if(is.null(times)){
         times <- x$times
     }else{
         if(any(times %in% x$times == FALSE)){
@@ -362,7 +403,6 @@ iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
         }
     }
     n.times <- length(times)
-
     ls.score <- lava::score(x, times = times, simplifies = FALSE, indiv = TRUE)
     ls.info <- lava::information(x, times = times, simplifies = FALSE)
 
@@ -384,7 +424,7 @@ iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
         ##                  - n(\int W(0)) X(O) d\pi(0,\beta)/d\beta d\Prob_h(O)) * (d\beta(\Prob_h)/d\Prob_h)
         ##                  + n\int W(0) X(O) (Y(O) - \pi(O)) d(d\Prob_n(O)/dh)
         ## \Prob_h = (1-h)\Prob + h \delta_{O_i}
-        ## dS(\Prob_h)/dh|h = 0 = n\int IF_{W_O}(O_i) X(O) (Y(O) - \pi(O)) d\Prob_h(O) \IF_\eta(O_i)
+        ## dS(\Prob_h)/dh|h = 0 = n\int IF_{W_O}(O_i) X(O) (Y(O) - \pi(O)) d\Prob_h(O) 
         ##                        + dS/d\beta IF_\beta(O_i)
         ##                        + (-S + S_i) where S=0 (since it is at ML)
         ## IF_\beta(O_i) = (S_i + n\int IF_{W_O}(O_i) X(O) (Y(O) - \pi(O)) d\Prob_h(O)) / (-dS/d\beta)
@@ -392,16 +432,17 @@ iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
         ## NOTE: IF_{W_O}(O_i) = dW(O,\eta)/d\eta \IF_\eta(O_i)
         if(x$n.censor[iTime]>0){
             n.obs <- stats::nobs(iObject)
+            W2 <- iObject$prior.weights2
             X <- stats::model.matrix(iObject)
             pi <- stats::predict(iObject, type = "response")
             Y <- iObject$y
 
             factor <- TRUE        
-            attr(factor,"factor") <- lapply(apply(colMultiply_cpp(X, (Y - pi)), 2, list), function(iVec){cbind(iVec[[1]])})
+            attr(factor,"factor") <- lapply(apply(colMultiply_cpp(X, -W2*(Y - pi)), 2, list), function(iVec){cbind(iVec[[1]])})
             iPred <- predictRisk(x$cox, diag = TRUE, newdata = x$data, times = iObject$time.prior.weights,
-                                 type = "survival", product.limit = FALSE, average.iid = factor, store.iid = "full")
+                                 type = "survival", product.limit = FALSE, average.iid = factor, store.iid = "minimal")
         }
-        
+
         ## ** assemble uncertainty
         ## (S+dS/dW)/I
         if(x$n.censor[iTime]>0){
@@ -429,11 +470,10 @@ iid.wglm <- function(x, times = NULL, simplifies = TRUE, ...){
     }
 }
 
-
 ## * predictRisk.wglm
-#' @export
 #' @rdname predictRisk
 #' @method predictRisk wglm
+#' @export
 predictRisk.wglm <- function(object, newdata, times = NULL, 
                              product.limit = FALSE, diag = FALSE, iid = FALSE, average.iid = FALSE, ...){
 
