@@ -159,70 +159,3 @@ arma::mat rowMultiply_cpp(arma::mat X, const arma::rowvec& scale){
   return(X);
 }
 
-// * sliceMultiply_cpp (documentation)
-//' @title Apply * by slice
-//' @description Fast computation of sweep(X, MARGIN = 1:2, FUN = "*", STATS = scale)
-//' @name sliceMultiply_cpp
-//' 
-//' @param X An array.
-//' @param M A matrix with the same number of row and columns as X.
-//' 
-//' @return An array of same size as X.
-//' @author Brice Ozenne <broz@@sund.ku.dk>
-//' @examples
-//' x <- array(1, dim = c(2,6,5))
-//' M <- matrix(1:12,2,6)
-//' sweep(x, MARGIN = 1:2, FUN = "*", STATS = M)
-//' sliceMultiply_cpp(x, M) 
-//' 
-
-// * sliceMultiply_cpp (code)
-//' @rdname sliceMultiply_cpp
-//' @export
-// [[Rcpp::export]]
-arma::cube sliceMultiply_cpp(arma::cube X, const arma::mat& M){
-  arma::cube Y = X;
-  Y.each_slice() %= M;
-  return(Y);
-}
-
-//' @rdname sliceMultiply_cpp
-//' @export
-// [[Rcpp::export]]
-void sliceMultiplyPointer_cpp(arma::cube X, const arma::mat& M){
-  X.each_slice() %= M;
-}
-
-// * sliceScale_cpp (documentation)
-//' @title Apply / by slice
-//' @description Fast computation of sweep(X, MARGIN = 1:2, FUN = "/", STATS = scale)
-//' @name sliceScale_cpp
-//' 
-//' @param X An array.
-//' @param M A matrix with the same number of row and columns as X.
-//' 
-//' @return An array of same size as X.
-//' @author Brice Ozenne <broz@@sund.ku.dk>
-//' @examples
-//' x <- array(1, dim = c(2,6,5))
-//' M <- matrix(1:12,2,6)
-//' sweep(x, MARGIN = 1:2, FUN = "/", STATS = M)
-//' sliceScale_cpp(x, M) 
-//' 
-
-// * sliceScale_cpp (code)
-//' @rdname sliceScale_cpp
-//' @export
-// [[Rcpp::export]]
-arma::cube sliceScale_cpp(arma::cube X, const arma::mat& M){
-  arma::cube Y=X;
-  Y.each_slice() /= M;
-  return(Y);
-}
-
-//' @rdname sliceScale_cpp
-//' @export
-// [[Rcpp::export]]
-void sliceScalePointer_cpp(arma::cube X, const arma::mat& M){
-  X.each_slice() /= M;
-}
